@@ -22,16 +22,55 @@ setClass("extractedReadsListNonCombined", representation(identifiedReads = "extr
     structure(lapply(c("marker", "forwardflank", "reverseflank"), function(n) grep(n, tolower(colNames))), .Names = c("markerCol", "forwardCol", "reverseCol"))
 }
 
-## .mclapplyvmatchPattern and .vmatchMultiplePatternsAlternate are used for diagnostic purposes only
-.mclapplyvmatchPattern <- function(flanks, seqs, max.mismatch, numberOfThreads, limitSeqRange = NULL) {
+.mmatchPattern <- function(pattern, subject, max.mismatch, with.indels) {
+    xx <- lapply(seq_along(subject), function(j) {
+        matchPattern(pattern, subject[[j]],
+                     with.indels = with.indels,
+                     max.mismatch = max.mismatch,
+                     algorithm = "indels")
+    })
+
+    ans_ends <- lapply(xx, function(yy) yy@ranges@start + yy@ranges@width - 1L)
+    ans_width0 <- as.integer(sapply(seq_along(xx), function(yy) {
+        if (length(xx[[yy]]@ranges@width) > 0) {
+            res <- xx[[yy]]@ranges@width[which.max(ans_ends[[yy]])]
+        }
+        else {
+            res <- 0
+        }
+
+        res
+    }))
+
+    yy <- new("ByPos_MIndex", width0 = ans_width0, NAMES = names(subject), ends = ans_ends)
+    return(yy)
+}
+
+.mclapplymatchPattern <- function(flanks, seqs, max.mismatch, numberOfThreads,
+                                  limitSeqRange = NULL, with.indels = FALSE) {
+
+    if (with.indels) {
+        internalMatchPattern <- .mmatchPattern
+    }
+    else {
+        internalMatchPattern <- vmatchPattern
+    }
+
     if (is.null(limitSeqRange)) {
-        resList <- mclapply(seq_along(flanks), function(f) vmatchPattern(flanks[[f]], seqs, with.indels = FALSE, max.mismatch = max.mismatch),
-                            mc.cores = numberOfThreads)
+        resList <- mclapply(seq_along(flanks), function(f) {
+            internalMatchPattern(pattern = flanks[[f]],
+                                 subject = seqs,
+                                 with.indels = with.indels,
+                                 max.mismatch = max.mismatch)
+        }, mc.cores = numberOfThreads)
     }
     else {
         resList <- mclapply(seq_along(flanks), function(f) {
             if (!is.null(limitSeqRange[[f]])) {
-                res <- vmatchPattern(flanks[[f]], seqs[limitSeqRange[[f]]], with.indels = FALSE, max.mismatch = max.mismatch)
+                res <- internalMatchPattern(pattern = flanks[[f]],
+                                            subject = seqs[limitSeqRange[[f]]],
+                                            with.indels = with.indels,
+                                            max.mismatch = max.mismatch)
                 return(res)
             }
             else {
@@ -42,35 +81,53 @@ setClass("extractedReadsListNonCombined", representation(identifiedReads = "extr
     return(resList)
 }
 
-.identifyFlankingRegions <- function(seqs, flankingRegions, colList, numberOfMutation = 1, numberOfThreads = 4, removeEmptyMarkers = TRUE,
-                                     remainingSequences = NULL) {
-    if (!is.null(remainingSequences))
-        if (!is.list(remainingSequences))
+
+.identifyFlankingRegions <- function(seqs, flankingRegions, colList, numberOfMutation = 1, numberOfThreads = 4,
+                                     removeEmptyMarkers = TRUE, remainingSequences = NULL,
+                                     indels = FALSE) {
+    if (!is.null(remainingSequences)) {
+        if (!is.list(remainingSequences)) {
             remainingSequences = rep(list(remainingSequences), times = length(flankingRegions$Marker))
+        }
+    }
 
     forwardFlank <- DNAStringSet(unname(unlist(flankingRegions[, colList$forwardCol])))
     reverseFlank <- DNAStringSet(unname(unlist(flankingRegions[, colList$reverseCol])))
 
     # Forward match
-    identifiedForward <- .mclapplyvmatchPattern(forwardFlank, seqs, numberOfMutation, numberOfThreads, limitSeqRange = remainingSequences)
+    identifiedForward <- .mclapplymatchPattern(flanks = forwardFlank,
+                                               seqs = seqs,
+                                               max.mismatch = numberOfMutation,
+                                               numberOfThreads = numberOfThreads,
+                                               limitSeqRange = remainingSequences,
+                                               with.indels = indels)
     forwardMatch <- lapply(1:nrow(flankingRegions), function(i) which(elementNROWS(identifiedForward[[i]]) >= 1L))
 
     # Reverse match
-    identifiedReverse <- .mclapplyvmatchPattern(reverseFlank, seqs, numberOfMutation, numberOfThreads, limitSeqRange = forwardMatch)
+    identifiedReverse <- .mclapplymatchPattern(flanks = reverseFlank,
+                                               seqs = seqs,
+                                               max.mismatch = numberOfMutation,
+                                               numberOfThreads = numberOfThreads,
+                                               limitSeqRange = forwardMatch,
+                                               with.indels = indels)
     reverseMatch <- lapply(1:nrow(flankingRegions), function(i) which(elementNROWS(identifiedReverse[[i]]) >= 1L))
 
     matchedSeq <- lapply(seq_along(reverseMatch), function(i) forwardMatch[[i]][reverseMatch[[i]]])
-
     if(removeEmptyMarkers) {
         nonEmptyEntries <- which(sapply(matchedSeq, length) != 0)
         rList <- list(markers = unname(unlist(flankingRegions[nonEmptyEntries, colList$markerCol])),
-                      identifiedForward = identifiedForward[nonEmptyEntries], identifiedReverse = identifiedReverse[nonEmptyEntries],
-                      matchedSeq = matchedSeq[nonEmptyEntries], reverseMatchedSeq = reverseMatch[nonEmptyEntries],
+                      identifiedForward = identifiedForward[nonEmptyEntries],
+                      identifiedReverse = identifiedReverse[nonEmptyEntries],
+                      matchedSeq = matchedSeq[nonEmptyEntries],
+                      reverseMatchedSeq = reverseMatch[nonEmptyEntries],
                       remainingSequences = seq_along(seqs)[-unique(unlist(matchedSeq, recursive = TRUE))])
     }
     else {
-        rList <- list(markers = unname(unlist((flankingRegions[, colList$markerCol]))), identifiedForward = identifiedForward,
-                      identifiedReverse = identifiedReverse, matchedSeq = matchedSeq, reverseMatchedSeq = reverseMatch,
+        rList <- list(markers = unname(unlist((flankingRegions[, colList$markerCol]))),
+                      identifiedForward = identifiedForward,
+                      identifiedReverse = identifiedReverse,
+                      matchedSeq = matchedSeq,
+                      reverseMatchedSeq = reverseMatch,
                       remainingSequences = seq_along(seqs)[-unique(unlist(matchedSeq, recursive = TRUE))])
     }
 
@@ -81,12 +138,18 @@ setClass("extractedReadsListNonCombined", representation(identifiedReads = "extr
 #####
 .extractAndTrimMarkerIdentifiedReads.mclapply <- function(seqs, qual, flankingRegions, colList, numberOfMutation, removeEmptyMarkers,
                                                           flankSizes, numberOfThreads = 4, reversed = FALSE, reverseComplementRun = FALSE,
-                                                          remainingSequences = NULL) {
-    identifiedFlanksObj <- .identifyFlankingRegions(seqs = seqs, flankingRegions = flankingRegions, colList = colList,
-                                                        numberOfMutation = numberOfMutation, numberOfThreads = numberOfThreads,
-                                                        removeEmptyMarkers = removeEmptyMarkers, remainingSequences = remainingSequences)
+                                                          remainingSequences = NULL,
+                                                          indels = FALSE) {
+    identifiedFlanksObj <- .identifyFlankingRegions(seqs = seqs,
+                                                    flankingRegions = flankingRegions,
+                                                    colList = colList,
+                                                    numberOfMutation = numberOfMutation,
+                                                    numberOfThreads = numberOfThreads,
+                                                    removeEmptyMarkers = removeEmptyMarkers,
+                                                    remainingSequences = remainingSequences,
+                                                    indels = FALSE)
 
-    whichMarkers = which(unlist(flankingRegions[, colList$markerCol]) %in% identifiedFlanksObj$markers)
+    whichMarkers <- which(unlist(flankingRegions[, colList$markerCol]) %in% identifiedFlanksObj$markers)
     identifiedMarkers <- structure(mclapply(seq_along(identifiedFlanksObj$matchedSeq), function(i) {
         marker <- identifiedFlanksObj$markers[i]
         identifiedForward <- identifiedFlanksObj$identifiedForward[[i]]
@@ -95,12 +158,14 @@ setClass("extractedReadsListNonCombined", representation(identifiedReads = "extr
         reverseMatchedSeq <- identifiedFlanksObj$reverseMatchedSeq[[i]]
 
         # F: Take the first in the IRanges
-        endForward <- unlist(lapply(endIndex(identifiedForward)[matchedSeq], function(v) v[1L]))
-        startForward <- endForward - (flankSizes[whichMarkers[i], 1] - 1)
+        endForward <- unlist(lapply(endIndex(identifiedForward)[matchedSeq], function(v) min(v)))
+        widthForward <- unlist(lapply(width(identifiedForward)[matchedSeq], function(v) unique(v)))
+        startForward <- endForward - (widthForward - 1)
 
         # R: Take the last in the IRanges
-        startReverse <- unlist(lapply(startIndex(identifiedReverse)[reverseMatchedSeq], function(v) v[length(v)]))
-        endReverse <- startReverse + (flankSizes[whichMarkers[i], 2] - 1)
+        startReverse <- unlist(lapply(startIndex(identifiedReverse)[reverseMatchedSeq], function(v) max(v)))
+        widthReverse <- unlist(lapply(width(identifiedReverse)[reverseMatchedSeq], function(v) unique(v)))
+        endReverse <- startReverse + (widthReverse - 1)
 
         # Removes reads where forward is NOT observed before reverse
         keepSeq <- which(startReverse > (endForward + 1) & endReverse <= nchar(seqs[matchedSeq]) & startForward > 0)
@@ -116,10 +181,10 @@ setClass("extractedReadsListNonCombined", representation(identifiedReads = "extr
             rList <- NULL
         }
         else {
-            trimmedIncludingFlanks <- subseq(seqs[matchedSeq], start = startForward, end = endReverse)
-            trimmedQualityIncludingFlanks <- subseq(quality(qual[matchedSeq]), start = startForward, end = endReverse)
-
             if (!(reversed && reverseComplementRun)) {
+                trimmedIncludingFlanks <- subseq(seqs[matchedSeq], start = startForward, end = endReverse)
+                trimmedQualityIncludingFlanks <- subseq(quality(qual[matchedSeq]), start = startForward, end = endReverse)
+
                 forwardFlankingRegion <- subseq(seqs[matchedSeq], start = startForward, end = endForward + 1)
                 reverseFlankingRegion <- subseq(seqs[matchedSeq], start = startReverse - 1, end = endReverse)
 
@@ -131,6 +196,9 @@ setClass("extractedReadsListNonCombined", representation(identifiedReads = "extr
                 trimmedQuality <- subseq(quality(qual[matchedSeq]), start = endForward + 1, end = startReverse - 1)
             }
             else {
+                trimmedIncludingFlanks <- reverseComplement(subseq(seqs[matchedSeq], start = startForward, end = endReverse))
+                trimmedQualityIncludingFlanks <- reverse(subseq(quality(qual[matchedSeq]), start = startForward, end = endReverse))
+
                 forwardFlankingRegion <- reverseComplement(subseq(seqs[matchedSeq], start = startReverse - 1, end = endReverse))
                 reverseFlankingRegion <- reverseComplement(subseq(seqs[matchedSeq], start = startForward, end = endForward + 1))
 
@@ -156,7 +224,7 @@ setClass("extractedReadsListNonCombined", representation(identifiedReads = "extr
         return(rList)
     }, mc.cores = numberOfThreads), .Names = as.character(identifiedFlanksObj$markers)) #
 
-    # Remove sequences occuring at several loci
+    # Remove sequences occurring at several loci
     identifiedSeqTable <- sort(table(unlist(lapply(identifiedMarkers, function(r) r$matchedSeq))), decreasing = TRUE)
     identifiedSeqNotUnique <- as.integer(names(identifiedSeqTable[identifiedSeqTable > 1L]))
     identifiedMarkersUniqueSequences <- lapply(identifiedMarkers, function(r) {
@@ -167,19 +235,33 @@ setClass("extractedReadsListNonCombined", representation(identifiedReads = "extr
                     trimmed = r$trimmed[keepSeq, ], trimmedQuality = r$trimmedQuality[keepSeq, ]))
     })
 
-    extractedReadsList <- list(n_reads = sum(sapply(identifiedMarkersUniqueSequences, function(vv) length(vv[["matchedSeq"]]))),
-                               reverseComplement = reverseComplementRun, identifiedMarkers = identifiedMarkers,
+    if (length(identifiedMarkersUniqueSequences) > 0) {
+        n_reads <- sum(sapply(identifiedMarkersUniqueSequences, function(vv) length(vv[["matchedSeq"]])))
+    }
+    else {
+        n_reads <- 0
+    }
+
+    extractedReadsList <- list(n_reads = n_reads,
+                               reverseComplement = reverseComplementRun,
+                               identifiedMarkers = identifiedMarkers,
                                identifiedMarkersSequencesUniquelyAssigned = identifiedMarkersUniqueSequences,
                                remainingSequences = identifiedFlanksObj$remainingSequences)
     class(extractedReadsList) <- "extractedReadsList"
     return(extractedReadsList)
 }
 
-.extractAndTrimMarkerIdentifiedReadsReverseComplement <- function(seqs, qual, flankingRegions, colList = NULL,
-                                                                  numberOfMutation, removeEmptyMarkers = TRUE,
-                                                                  numberOfThreads = numberOfThreads, reversed = TRUE,
+.extractAndTrimMarkerIdentifiedReadsReverseComplement <- function(seqs,
+                                                                  qual,
+                                                                  flankingRegions,
+                                                                  colList = NULL,
+                                                                  numberOfMutation,
+                                                                  removeEmptyMarkers = TRUE,
+                                                                  numberOfThreads = numberOfThreads,
+                                                                  reversed = TRUE,
                                                                   matchPatternMethod = "mclapply",
-                                                                  remainingSequences = NULL) {
+                                                                  remainingSequences = NULL,
+                                                                  indels = FALSE) {
 
     flankingRegionsReverseComplement <- structure(data.frame(flankingRegions[, colList$markerCol],
                                                              unname(as.character(reverseComplement(DNAStringSet(as_vector(flankingRegions[, colList$reverseCol]))))),
@@ -190,13 +272,18 @@ setClass("extractedReadsListNonCombined", representation(identifiedReads = "extr
     extractAndTrimMarkerIdentifiedReads = switch(matchPatternMethod,
                                                  "mclapply" = .extractAndTrimMarkerIdentifiedReads.mclapply)
 
-    extractedTrimmedRC <- extractAndTrimMarkerIdentifiedReads(seqs, qual, flankingRegionsReverseComplement,
+    extractedTrimmedRC <- extractAndTrimMarkerIdentifiedReads(seqs = seqs,
+                                                              qual = qual,
+                                                              flankingRegions = flankingRegionsReverseComplement,
                                                               colList = list(markerCol = 1, forwardCol = 2, reverseCol = 3),
-                                                              numberOfMutation = numberOfMutation, removeEmptyMarkers = removeEmptyMarkers,
-                                                              flankSizes = apply(flankingRegionsReverseComplement[, -1], 2, nchar),
+                                                              numberOfMutation = numberOfMutation,
+                                                              removeEmptyMarkers = removeEmptyMarkers,
+                                                              flankSizes = matrix(apply(flankingRegionsReverseComplement[, -1], 2, nchar), ncol = 2),
                                                               numberOfThreads = numberOfThreads,
-                                                              reversed = reversed, reverseComplementRun = TRUE,
-                                                              remainingSequences = remainingSequences)
+                                                              reversed = reversed,
+                                                              reverseComplementRun = TRUE,
+                                                              remainingSequences = remainingSequences,
+                                                              indels = indels)
 
     class(extractedTrimmedRC) <- "extractedReadsListReverseComplement"
     return(extractedTrimmedRC)
@@ -233,57 +320,82 @@ setClass("extractedReadsListNonCombined", representation(identifiedReads = "extr
 #' @param combineLists TRUE/FALSE: If 'includeReverseComplement' is TRUE, should the sets be combined?
 #' @param removeEmptyMarkers TRUE/FALSE: Should markers returning no identified regions be removed?
 #' @param matchPatternMethod Which method should be used to identify the flanking regions (only 'mclapply' implemented at the moment)?
+#' @param indels TRUE/FALSE: Allow indels as well as base mismatches?
+#' @param trace TRUE/FALSE: Show trace? Note: only used if bam list is supplied.
 #'
 #' @return A control list setting default behaviour.
 identifySTRRegions.control <- function(colList = NULL, numberOfThreads = 4L, reversed = TRUE,
                                        includeReverseComplement = TRUE, combineLists = TRUE, removeEmptyMarkers = TRUE,
-                                       matchPatternMethod = "mclapply") {
+                                       matchPatternMethod = "mclapply", indels = FALSE, trace = FALSE) {
     controlList <- list(colList = NULL, numberOfThreads = numberOfThreads, removeEmptyMarkers = removeEmptyMarkers,
                         reversed = reversed, includeReverseComplement = includeReverseComplement,
-                        combineLists = combineLists, matchPatternMethod = matchPatternMethod)
+                        combineLists = combineLists, matchPatternMethod = matchPatternMethod,
+                        indels = indels, trace = trace)
     return(controlList)
 }
 
 
 .ShortReadQ.identifySTRRegions <- function(reads, flankingRegions, numberOfMutation, control = identifySTRRegions.control()) {
-    seqs <- ShortRead::sread(reads)
-    qual <- Biostrings::quality(reads)
+    seqs <- sread(reads)
+    qual <- quality(reads)
 
     colID <- if(is.null(control$colList)) .getCols(names(flankingRegions)) else control$colList
-    flankSizes <- apply(flankingRegions[, c(colID$forwardCol, colID$reverseCol)], 2, nchar)
+    flankSizes <- matrix(apply(flankingRegions[, c(colID$forwardCol, colID$reverseCol)], 2, nchar), ncol = 2)
 
     extractAndTrimMarkerIdentifiedReads = switch(control$matchPatternMethod,
                                                  "mclapply" = .extractAndTrimMarkerIdentifiedReads.mclapply)
 
-    extractedSTRs <- extractAndTrimMarkerIdentifiedReads(seqs = seqs, qual = qual, flankingRegions = flankingRegions,
-                                                         colList = colID, numberOfMutation = numberOfMutation,
-                                                         removeEmptyMarkers = control$removeEmptyMarkers, flankSizes = flankSizes,
-                                                         numberOfThreads = control$numberOfThreads, reversed = FALSE,
-                                                         reverseComplementRun = FALSE, remainingSequences = NULL)
+    extractedSTRs <- extractAndTrimMarkerIdentifiedReads(seqs = seqs,
+                                                         qual = qual,
+                                                         flankingRegions = flankingRegions,
+                                                         colList = colID,
+                                                         numberOfMutation = numberOfMutation,
+                                                         removeEmptyMarkers = control$removeEmptyMarkers,
+                                                         flankSizes = flankSizes,
+                                                         numberOfThreads = control$numberOfThreads,
+                                                         reversed = FALSE,
+                                                         reverseComplementRun = FALSE,
+                                                         remainingSequences = NULL,
+                                                         indels = control$indels)
 
     if (control$includeReverseComplement & (length(extractedSTRs$remainingSequences) > 0)) {
-        extractedSTRs_RC <- .extractAndTrimMarkerIdentifiedReadsReverseComplement(seqs, qual, flankingRegions, colList = colID,
+        extractedSTRs_RC <- .extractAndTrimMarkerIdentifiedReadsReverseComplement(seqs = seqs,
+                                                                                  qual = qual,
+                                                                                  flankingRegions = flankingRegions,
+                                                                                  colList = colID,
                                                                                   numberOfMutation = numberOfMutation,
                                                                                   removeEmptyMarkers = control$removeEmptyMarkers,
                                                                                   numberOfThreads = control$numberOfThreads,
                                                                                   reversed = control$reversed,
                                                                                   matchPatternMethod = control$matchPatternMethod,
-                                                                                  remainingSequences = NULL) #extractedSTRs$remainingSequences)
+                                                                                  remainingSequences = NULL,
+                                                                                  indels = control$indels) #extractedSTRs$remainingSequences)
 
         if (control$combineLists) {
             combinedRegions <- .combineMarkerIdentifiedReadsLists(extractedSTRs[which(names(extractedSTRs) != "remainingSequences")],
                                                                   extractedSTRs_RC[which(names(extractedSTRs) != "remainingSequences")])
-            return(combinedRegions)
+            res <- combinedRegions
         }
         else {
             extractedSTRs_Dual <- list(identifiedReads = extractedSTRs, identifiedReverseComplementReads = extractedSTRs_RC)
             class(extractedSTRs_Dual) <- "extractedReadsListNonCombined"
-            return(extractedSTRs_Dual)
+            res <- extractedSTRs_Dual
         }
     }
     else {
-        return(extractedSTRs)
+        res <- extractedSTRs
     }
+
+    ## Corrects the order of the markers to match the 'flankingRegions' file...
+    if (!is(res, "extractedReadsListNonCombined")) {
+        sortedIncludedMarkers <- order(sapply(names(res$identifiedMarkers), function(m) which(m == flankingRegions$Marker)))
+        res$identifiedMarkers <- res$identifiedMarkers[sortedIncludedMarkers]
+
+        sortedIncludedMarkers <- order(sapply(names(res$identifiedMarkersSequencesUniquelyAssigned), function(m) which(m == flankingRegions$Marker)))
+        res$identifiedMarkersSequencesUniquelyAssigned <- res$identifiedMarkersSequencesUniquelyAssigned[sortedIncludedMarkers]
+    }
+
+    return(res)
 }
 
 .character.identifySTRRegions <- function(reads, flankingRegions, numberOfMutation, control = identifySTRRegions.control()) {
@@ -327,7 +439,6 @@ setGeneric("identifySTRRegions", signature = "reads",
 #' @return The returned object is a list of lists. If the reverse complement strings are not included or if the \code{control$combineLists == TRUE},
 #' a list, contains lists of untrimmed and trimmed strings for each row in \code{flankingRegions}. If \code{control$combineLists == FALSE}, the function returns a list of two such lists,
 #' one for forward strings and one for the reverse complement strings.
-#' @example inst/examples/identify.R
 setMethod("identifySTRRegions", "ShortReadQ",
           function(reads, flankingRegions, numberOfMutation = 1, control = identifySTRRegions.control())
               .ShortReadQ.identifySTRRegions(reads = reads, flankingRegions = flankingRegions, numberOfMutation = numberOfMutation, control = control)
@@ -347,7 +458,6 @@ setMethod("identifySTRRegions", "ShortReadQ",
 #' @return The returned object is a list of lists. If the reverse complement strings are not included or if the \code{control$combineLists == TRUE},
 #' a list, contains lists of untrimmed and trimmed strings for each row in \code{flankingRegions}. If \code{control$combineLists == FALSE}, the function returns a list of two such lists,
 #' one for forward strings and one for the reverse complement strings.
-#' @example inst/examples/identify.R
 setMethod("identifySTRRegions", "character",
           function(reads, flankingRegions, numberOfMutation = 1, control = identifySTRRegions.control())
               .character.identifySTRRegions(reads = reads, flankingRegions = flankingRegions, numberOfMutation = numberOfMutation, control = control)
